@@ -1,26 +1,36 @@
 """Config flow for Shelly integration."""
 from __future__ import annotations
 
-import asyncio
 from collections.abc import Mapping
-from http import HTTPStatus
 from typing import Any, Final
 
-import aiohttp
 import aioshelly
 from aioshelly.block_device import BlockDevice
+from aioshelly.exceptions import (
+    DeviceConnectionError,
+    FirmwareUnsupported,
+    InvalidAuthError,
+)
 from aioshelly.rpc_device import RpcDevice
-import async_timeout
+from awesomeversion import AwesomeVersion
 import voluptuous as vol
 
 from homeassistant import config_entries
 from homeassistant.components import zeroconf
 from homeassistant.const import CONF_HOST, CONF_PASSWORD, CONF_USERNAME
-from homeassistant.core import HomeAssistant
+from homeassistant.core import HomeAssistant, callback
 from homeassistant.data_entry_flow import FlowResult
-from homeassistant.helpers import aiohttp_client
+from homeassistant.helpers import aiohttp_client, selector
 
-from .const import AIOSHELLY_DEVICE_TIMEOUT_SEC, CONF_SLEEP_PERIOD, DOMAIN, LOGGER
+from .const import (
+    BLE_MIN_VERSION,
+    CONF_BLE_SCANNER_MODE,
+    CONF_SLEEP_PERIOD,
+    DOMAIN,
+    LOGGER,
+    BLEScannerMode,
+)
+from .coordinator import async_reconnect_soon, get_entry_data
 from .utils import (
     get_block_device_name,
     get_block_device_sleep_period,
@@ -29,11 +39,21 @@ from .utils import (
     get_info_gen,
     get_model_name,
     get_rpc_device_name,
+    get_rpc_device_sleep_period,
+    get_ws_context,
+    mac_address_from_name,
 )
 
 HOST_SCHEMA: Final = vol.Schema({vol.Required(CONF_HOST): str})
 
-HTTP_CONNECT_ERRORS: Final = (asyncio.TimeoutError, aiohttp.ClientError)
+
+BLE_SCANNER_OPTIONS = [
+    selector.SelectOptionDict(value=BLEScannerMode.DISABLED, label="Disabled"),
+    selector.SelectOptionDict(value=BLEScannerMode.ACTIVE, label="Active"),
+    selector.SelectOptionDict(value=BLEScannerMode.PASSIVE, label="Passive"),
+]
+
+INTERNAL_WIFI_AP_IP = "192.168.33.1"
 
 
 async def validate_input(
@@ -52,37 +72,38 @@ async def validate_input(
         data.get(CONF_PASSWORD),
     )
 
-    async with async_timeout.timeout(AIOSHELLY_DEVICE_TIMEOUT_SEC):
-        if get_info_gen(info) == 2:
-            rpc_device = await RpcDevice.create(
-                aiohttp_client.async_get_clientsession(hass),
-                options,
-            )
-            await rpc_device.shutdown()
-            assert rpc_device.shelly
-
-            return {
-                "title": get_rpc_device_name(rpc_device),
-                CONF_SLEEP_PERIOD: 0,
-                "model": rpc_device.shelly.get("model"),
-                "gen": 2,
-            }
-
-        # Gen1
-        coap_context = await get_coap_context(hass)
-        block_device = await BlockDevice.create(
+    if get_info_gen(info) == 2:
+        ws_context = await get_ws_context(hass)
+        rpc_device = await RpcDevice.create(
             aiohttp_client.async_get_clientsession(hass),
-            coap_context,
+            ws_context,
             options,
         )
-        block_device.shutdown()
+        await rpc_device.shutdown()
+        assert rpc_device.shelly
+
         return {
-            "title": get_block_device_name(block_device),
-            CONF_SLEEP_PERIOD: get_block_device_sleep_period(block_device.settings),
-            "model": block_device.model,
-            "gen": 1,
+            "title": get_rpc_device_name(rpc_device),
+            CONF_SLEEP_PERIOD: get_rpc_device_sleep_period(rpc_device.config),
+            "model": rpc_device.shelly.get("model"),
+            "gen": 2,
         }
 
+    # Gen1
+    coap_context = await get_coap_context(hass)
+    block_device = await BlockDevice.create(
+        aiohttp_client.async_get_clientsession(hass),
+        coap_context,
+        options,
+    )
+    block_device.shutdown()
+    return {
+        "title": get_block_device_name(block_device),
+        CONF_SLEEP_PERIOD: get_block_device_sleep_period(block_device.settings),
+        "model": block_device.model,
+        "gen": 1,
+    }
+
 
 class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
     """Handle a config flow for Shelly."""
@@ -103,9 +124,9 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
             host: str = user_input[CONF_HOST]
             try:
                 self.info = await self._async_get_info(host)
-            except HTTP_CONNECT_ERRORS:
+            except DeviceConnectionError:
                 errors["base"] = "cannot_connect"
-            except aioshelly.exceptions.FirmwareUnsupported:
+            except FirmwareUnsupported:
                 return self.async_abort(reason="unsupported_firmware")
             except Exception:  # pylint: disable=broad-except
                 LOGGER.exception("Unexpected exception")
@@ -121,7 +142,7 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
                     device_info = await validate_input(
                         self.hass, self.host, self.info, {}
                     )
-                except HTTP_CONNECT_ERRORS:
+                except DeviceConnectionError:
                     errors["base"] = "cannot_connect"
                 except Exception:  # pylint: disable=broad-except
                     LOGGER.exception("Unexpected exception")
@@ -155,16 +176,9 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
                 device_info = await validate_input(
                     self.hass, self.host, self.info, user_input
                 )
-            except aiohttp.ClientResponseError as error:
-                if error.status == HTTPStatus.UNAUTHORIZED:
-                    errors["base"] = "invalid_auth"
-                else:
-                    errors["base"] = "cannot_connect"
-            except aioshelly.exceptions.InvalidAuthError:
+            except InvalidAuthError:
                 errors["base"] = "invalid_auth"
-            except HTTP_CONNECT_ERRORS:
-                errors["base"] = "cannot_connect"
-            except aioshelly.exceptions.JSONRPCError:
+            except DeviceConnectionError:
                 errors["base"] = "cannot_connect"
             except Exception:  # pylint: disable=broad-except
                 LOGGER.exception("Unexpected exception")
@@ -199,22 +213,48 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
             step_id="credentials", data_schema=vol.Schema(schema), errors=errors
         )
 
+    async def _async_discovered_mac(self, mac: str, host: str) -> None:
+        """Abort and reconnect soon if the device with the mac address is already configured."""
+        if (
+            current_entry := await self.async_set_unique_id(mac)
+        ) and current_entry.data[CONF_HOST] == host:
+            await async_reconnect_soon(self.hass, current_entry)
+        if host == INTERNAL_WIFI_AP_IP:
+            # If the device is broadcasting the internal wifi ap ip
+            # we can't connect to it, so we should not update the
+            # entry with the new host as it will be unreachable
+            #
+            # This is a workaround for a bug in the firmware 0.12 (and older?)
+            # which should be removed once the firmware is fixed
+            # and the old version is no longer in use
+            self._abort_if_unique_id_configured()
+        else:
+            self._abort_if_unique_id_configured({CONF_HOST: host})
+
     async def async_step_zeroconf(
         self, discovery_info: zeroconf.ZeroconfServiceInfo
     ) -> FlowResult:
         """Handle zeroconf discovery."""
         host = discovery_info.host
+        # First try to get the mac address from the name
+        # so we can avoid making another connection to the
+        # device if we already have it configured
+        if mac := mac_address_from_name(discovery_info.name):
+            await self._async_discovered_mac(mac, host)
+
         try:
             self.info = await self._async_get_info(host)
-        except HTTP_CONNECT_ERRORS:
+        except DeviceConnectionError:
             return self.async_abort(reason="cannot_connect")
-        except aioshelly.exceptions.FirmwareUnsupported:
+        except FirmwareUnsupported:
             return self.async_abort(reason="unsupported_firmware")
 
-        await self.async_set_unique_id(self.info["mac"])
-        self._abort_if_unique_id_configured({CONF_HOST: host})
-        self.host = host
+        if not mac:
+            # We could not get the mac address from the name
+            # so need to check here since we just got the info
+            await self._async_discovered_mac(self.info["mac"], host)
 
+        self.host = host
         self.context.update(
             {
                 "title_placeholders": {"name": discovery_info.name.split(".")[0]},
@@ -227,7 +267,7 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
 
         try:
             self.device_info = await validate_input(self.hass, self.host, self.info, {})
-        except HTTP_CONNECT_ERRORS:
+        except DeviceConnectionError:
             return self.async_abort(reason="cannot_connect")
 
         return await self.async_step_confirm_discovery()
@@ -280,23 +320,14 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         if user_input is not None:
             try:
                 info = await self._async_get_info(host)
-            except (
-                asyncio.TimeoutError,
-                aiohttp.ClientError,
-                aioshelly.exceptions.FirmwareUnsupported,
-            ):
+            except (DeviceConnectionError, InvalidAuthError, FirmwareUnsupported):
                 return self.async_abort(reason="reauth_unsuccessful")
 
             if self.entry.data.get("gen", 1) != 1:
                 user_input[CONF_USERNAME] = "admin"
             try:
                 await validate_input(self.hass, host, info, user_input)
-            except (
-                aiohttp.ClientResponseError,
-                aioshelly.exceptions.InvalidAuthError,
-                asyncio.TimeoutError,
-                aiohttp.ClientError,
-            ):
+            except (DeviceConnectionError, InvalidAuthError, FirmwareUnsupported):
                 return self.async_abort(reason="reauth_unsuccessful")
             else:
                 self.hass.config_entries.async_update_entry(
@@ -321,7 +352,64 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
 
     async def _async_get_info(self, host: str) -> dict[str, Any]:
         """Get info from shelly device."""
-        async with async_timeout.timeout(AIOSHELLY_DEVICE_TIMEOUT_SEC):
-            return await aioshelly.common.get_info(
-                aiohttp_client.async_get_clientsession(self.hass), host
-            )
+        return await aioshelly.common.get_info(
+            aiohttp_client.async_get_clientsession(self.hass), host
+        )
+
+    @staticmethod
+    @callback
+    def async_get_options_flow(
+        config_entry: config_entries.ConfigEntry,
+    ) -> OptionsFlowHandler:
+        """Get the options flow for this handler."""
+        return OptionsFlowHandler(config_entry)
+
+    @classmethod
+    @callback
+    def async_supports_options_flow(
+        cls, config_entry: config_entries.ConfigEntry
+    ) -> bool:
+        """Return options flow support for this handler."""
+        return config_entry.data.get("gen") == 2 and not config_entry.data.get(
+            CONF_SLEEP_PERIOD
+        )
+
+
+class OptionsFlowHandler(config_entries.OptionsFlow):
+    """Handle the option flow for shelly."""
+
+    def __init__(self, config_entry: config_entries.ConfigEntry) -> None:
+        """Initialize options flow."""
+        self.config_entry = config_entry
+
+    async def async_step_init(
+        self, user_input: dict[str, Any] | None = None
+    ) -> FlowResult:
+        """Handle options flow."""
+        if user_input is not None:
+            entry_data = get_entry_data(self.hass)[self.config_entry.entry_id]
+            if user_input[CONF_BLE_SCANNER_MODE] != BLEScannerMode.DISABLED and (
+                not entry_data.rpc
+                or AwesomeVersion(entry_data.rpc.device.version) < BLE_MIN_VERSION
+            ):
+                return self.async_abort(
+                    reason="ble_unsupported",
+                    description_placeholders={"ble_min_version": BLE_MIN_VERSION},
+                )
+            return self.async_create_entry(title="", data=user_input)
+
+        return self.async_show_form(
+            step_id="init",
+            data_schema=vol.Schema(
+                {
+                    vol.Required(
+                        CONF_BLE_SCANNER_MODE,
+                        default=self.config_entry.options.get(
+                            CONF_BLE_SCANNER_MODE, BLEScannerMode.DISABLED
+                        ),
+                    ): selector.SelectSelector(
+                        selector.SelectSelectorConfig(options=BLE_SCANNER_OPTIONS),
+                    ),
+                }
+            ),
+        )
