@@ -2,9 +2,10 @@
 from __future__ import annotations
 
 import asyncio
+from collections.abc import Mapping
 from datetime import datetime, timedelta
 import logging
-from typing import Final, TypedDict
+from typing import Any, Final
 
 import aiohttp
 import async_timeout
@@ -27,25 +28,43 @@ from homeassistant.components.weather import (
     ATTR_CONDITION_WINDY,
     ATTR_CONDITION_WINDY_VARIANT,
     ATTR_FORECAST_CONDITION,
-    ATTR_FORECAST_PRECIPITATION,
-    ATTR_FORECAST_TEMP,
-    ATTR_FORECAST_TEMP_LOW,
+    ATTR_FORECAST_NATIVE_PRECIPITATION,
+    ATTR_FORECAST_NATIVE_PRESSURE,
+    ATTR_FORECAST_NATIVE_TEMP,
+    ATTR_FORECAST_NATIVE_TEMP_LOW,
+    ATTR_FORECAST_NATIVE_WIND_SPEED,
     ATTR_FORECAST_TIME,
+    ATTR_FORECAST_WIND_BEARING,
+    ROUNDING_PRECISION,
     Forecast,
     WeatherEntity,
 )
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import CONF_LATITUDE, CONF_LONGITUDE, CONF_NAME, TEMP_CELSIUS
+from homeassistant.const import (
+    CONF_LATITUDE,
+    CONF_LOCATION,
+    CONF_LONGITUDE,
+    CONF_NAME,
+    UnitOfLength,
+    UnitOfPrecipitationDepth,
+    UnitOfPressure,
+    UnitOfSpeed,
+    UnitOfTemperature,
+)
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers import aiohttp_client
+from homeassistant.helpers.device_registry import DeviceEntryType
+from homeassistant.helpers.entity import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.event import async_call_later
 from homeassistant.util import Throttle, slugify
+from homeassistant.util.unit_conversion import SpeedConverter
 
 from .const import (
     ATTR_SMHI_CLOUDINESS,
     ATTR_SMHI_THUNDER_PROBABILITY,
     ATTR_SMHI_WIND_GUST_SPEED,
+    DOMAIN,
     ENTITY_ID_SENSOR_FORMAT,
 )
 
@@ -69,6 +88,7 @@ CONDITION_CLASSES: Final[dict[str, list[int]]] = {
     ATTR_CONDITION_EXCEPTIONAL: [],
 }
 
+TIMEOUT = 10
 # 5 minutes between retrying connect to API again
 RETRY_TIMEOUT = 5 * 60
 
@@ -88,8 +108,8 @@ async def async_setup_entry(
 
     entity = SmhiWeather(
         location[CONF_NAME],
-        location[CONF_LATITUDE],
-        location[CONF_LONGITUDE],
+        location[CONF_LOCATION][CONF_LATITUDE],
+        location[CONF_LOCATION][CONF_LONGITUDE],
         session=session,
     )
     entity.entity_id = ENTITY_ID_SENSOR_FORMAT.format(name)
@@ -100,6 +120,15 @@ async def async_setup_entry(
 class SmhiWeather(WeatherEntity):
     """Representation of a weather entity."""
 
+    _attr_attribution = "Swedish weather institute (SMHI)"
+    _attr_native_temperature_unit = UnitOfTemperature.CELSIUS
+    _attr_native_visibility_unit = UnitOfLength.KILOMETERS
+    _attr_native_precipitation_unit = UnitOfPrecipitationDepth.MILLIMETERS
+    _attr_native_wind_speed_unit = UnitOfSpeed.METERS_PER_SECOND
+    _attr_native_pressure_unit = UnitOfPressure.HPA
+
+    _attr_has_entity_name = True
+
     def __init__(
         self,
         name: str,
@@ -108,32 +137,66 @@ class SmhiWeather(WeatherEntity):
         session: aiohttp.ClientSession,
     ) -> None:
         """Initialize the SMHI weather entity."""
-
-        self._name = name
-        self._latitude = latitude
-        self._longitude = longitude
+        self._attr_unique_id = f"{latitude}, {longitude}"
         self._forecasts: list[SmhiForecast] | None = None
         self._fail_count = 0
-        self._smhi_api = Smhi(self._longitude, self._latitude, session=session)
+        self._smhi_api = Smhi(longitude, latitude, session=session)
+        self._attr_device_info = DeviceInfo(
+            entry_type=DeviceEntryType.SERVICE,
+            identifiers={(DOMAIN, f"{latitude}, {longitude}")},
+            manufacturer="SMHI",
+            model="v2",
+            name=name,
+            configuration_url="http://opendata.smhi.se/apidocs/metfcst/parameters.html",
+        )
+        self._attr_condition = None
+        self._attr_native_temperature = None
 
     @property
-    def unique_id(self) -> str:
-        """Return a unique id."""
-        return f"{self._latitude}, {self._longitude}"
+    def extra_state_attributes(self) -> Mapping[str, Any] | None:
+        """Return additional attributes."""
+        if self._forecasts:
+            wind_gust = SpeedConverter.convert(
+                self._forecasts[0].wind_gust,
+                UnitOfSpeed.METERS_PER_SECOND,
+                self._wind_speed_unit,
+            )
+            return {
+                ATTR_SMHI_CLOUDINESS: self._forecasts[0].cloudiness,
+                ATTR_SMHI_WIND_GUST_SPEED: round(wind_gust, ROUNDING_PRECISION),
+                ATTR_SMHI_THUNDER_PROBABILITY: self._forecasts[0].thunder,
+            }
+        return None
 
     @Throttle(MIN_TIME_BETWEEN_UPDATES)
     async def async_update(self) -> None:
         """Refresh the forecast data from SMHI weather API."""
         try:
-            async with async_timeout.timeout(10):
-                self._forecasts = await self.get_weather_forecast()
+            async with async_timeout.timeout(TIMEOUT):
+                self._forecasts = await self._smhi_api.async_get_forecast()
                 self._fail_count = 0
-
         except (asyncio.TimeoutError, SmhiForecastException):
             _LOGGER.error("Failed to connect to SMHI API, retry in 5 minutes")
             self._fail_count += 1
             if self._fail_count < 3:
                 async_call_later(self.hass, RETRY_TIMEOUT, self.retry_update)
+                return
+
+        if self._forecasts:
+            self._attr_native_temperature = self._forecasts[0].temperature
+            self._attr_humidity = self._forecasts[0].humidity
+            self._attr_native_wind_speed = self._forecasts[0].wind_speed
+            self._attr_wind_bearing = self._forecasts[0].wind_direction
+            self._attr_native_visibility = self._forecasts[0].horizontal_visibility
+            self._attr_native_pressure = self._forecasts[0].pressure
+            self._attr_condition = next(
+                (
+                    k
+                    for k, v in CONDITION_CLASSES.items()
+                    if self._forecasts[0].symbol in v
+                ),
+                None,
+            )
 
     async def retry_update(self, _: datetime) -> None:
         """Retry refresh weather forecast."""
@@ -141,100 +204,6 @@ class SmhiWeather(WeatherEntity):
             no_throttle=True
         )
 
-    async def get_weather_forecast(self) -> list[SmhiForecast]:
-        """Return the current forecasts from SMHI API."""
-        return await self._smhi_api.async_get_forecast()
-
-    @property
-    def name(self) -> str:
-        """Return the name of the sensor."""
-        return self._name
-
-    @property
-    def temperature(self) -> int | None:
-        """Return the temperature."""
-        if self._forecasts is not None:
-            return self._forecasts[0].temperature
-        return None
-
-    @property
-    def temperature_unit(self) -> str:
-        """Return the unit of measurement."""
-        return TEMP_CELSIUS
-
-    @property
-    def humidity(self) -> int | None:
-        """Return the humidity."""
-        if self._forecasts is not None:
-            return self._forecasts[0].humidity
-        return None
-
-    @property
-    def wind_speed(self) -> float | None:
-        """Return the wind speed."""
-        if self._forecasts is not None:
-            # Convert from m/s to km/h
-            return round(self._forecasts[0].wind_speed * 18 / 5)
-        return None
-
-    @property
-    def wind_gust_speed(self) -> float | None:
-        """Return the wind gust speed."""
-        if self._forecasts is not None:
-            # Convert from m/s to km/h
-            return round(self._forecasts[0].wind_gust * 18 / 5)
-        return None
-
-    @property
-    def wind_bearing(self) -> int | None:
-        """Return the wind bearing."""
-        if self._forecasts is not None:
-            return self._forecasts[0].wind_direction
-        return None
-
-    @property
-    def visibility(self) -> float | None:
-        """Return the visibility."""
-        if self._forecasts is not None:
-            return self._forecasts[0].horizontal_visibility
-        return None
-
-    @property
-    def pressure(self) -> int | None:
-        """Return the pressure."""
-        if self._forecasts is not None:
-            return self._forecasts[0].pressure
-        return None
-
-    @property
-    def cloudiness(self) -> int | None:
-        """Return the cloudiness."""
-        if self._forecasts is not None:
-            return self._forecasts[0].cloudiness
-        return None
-
-    @property
-    def thunder_probability(self) -> int | None:
-        """Return the chance of thunder, unit Percent."""
-        if self._forecasts is not None:
-            return self._forecasts[0].thunder
-        return None
-
-    @property
-    def condition(self) -> str | None:
-        """Return the weather condition."""
-        if self._forecasts is None:
-            return None
-        return next(
-            (k for k, v in CONDITION_CLASSES.items() if self._forecasts[0].symbol in v),
-            None,
-        )
-
-    @property
-    def attribution(self) -> str:
-        """Return the attribution."""
-        return "Swedish weather institute (SMHI)"
-
     @property
     def forecast(self) -> list[Forecast] | None:
         """Return the forecast."""
@@ -251,31 +220,14 @@ class SmhiWeather(WeatherEntity):
             data.append(
                 {
                     ATTR_FORECAST_TIME: forecast.valid_time.isoformat(),
-                    ATTR_FORECAST_TEMP: forecast.temperature_max,
-                    ATTR_FORECAST_TEMP_LOW: forecast.temperature_min,
-                    ATTR_FORECAST_PRECIPITATION: round(forecast.total_precipitation, 1),
+                    ATTR_FORECAST_NATIVE_TEMP: forecast.temperature_max,
+                    ATTR_FORECAST_NATIVE_TEMP_LOW: forecast.temperature_min,
+                    ATTR_FORECAST_NATIVE_PRECIPITATION: forecast.total_precipitation,
                     ATTR_FORECAST_CONDITION: condition,
+                    ATTR_FORECAST_NATIVE_PRESSURE: forecast.pressure,
+                    ATTR_FORECAST_WIND_BEARING: forecast.wind_direction,
+                    ATTR_FORECAST_NATIVE_WIND_SPEED: forecast.wind_speed,
                 }
             )
 
         return data
-
-    @property
-    def extra_state_attributes(self) -> ExtraAttributes:
-        """Return SMHI specific attributes."""
-        extra_attributes: ExtraAttributes = {}
-        if self.cloudiness is not None:
-            extra_attributes[ATTR_SMHI_CLOUDINESS] = self.cloudiness
-        if self.wind_gust_speed is not None:
-            extra_attributes[ATTR_SMHI_WIND_GUST_SPEED] = self.wind_gust_speed
-        if self.thunder_probability is not None:
-            extra_attributes[ATTR_SMHI_THUNDER_PROBABILITY] = self.thunder_probability
-        return extra_attributes
-
-
-class ExtraAttributes(TypedDict, total=False):
-    """Represent the extra state attribute types."""
-
-    cloudiness: int
-    thunder_probability: int
-    wind_gust_speed: float
