@@ -7,8 +7,6 @@ from typing import NamedTuple
 
 import attr
 
-# mypy: disallow-any-generics
-
 
 class RGBColor(NamedTuple):
     """RGB hex values."""
@@ -296,22 +294,20 @@ def color_xy_brightness_to_RGB(
     b = X * 0.051713 - Y * 0.121364 + Z * 1.011530
 
     # Apply reverse gamma correction.
-    r, g, b = map(
-        lambda x: (12.92 * x)
-        if (x <= 0.0031308)
-        else ((1.0 + 0.055) * pow(x, (1.0 / 2.4)) - 0.055),
-        [r, g, b],
+    r, g, b = (
+        12.92 * x if (x <= 0.0031308) else ((1.0 + 0.055) * pow(x, (1.0 / 2.4)) - 0.055)
+        for x in (r, g, b)
     )
 
     # Bring all negative components to zero.
-    r, g, b = map(lambda x: max(0, x), [r, g, b])
+    r, g, b = (max(0, x) for x in (r, g, b))
 
     # If one component is greater than 1, weight components by that value.
     max_component = max(r, g, b)
     if max_component > 1:
-        r, g, b = map(lambda x: x / max_component, [r, g, b])
+        r, g, b = (x / max_component for x in (r, g, b))
 
-    ir, ig, ib = map(lambda x: int(x * 255), [r, g, b])
+    ir, ig, ib = (int(x * 255) for x in (r, g, b))
 
     return (ir, ig, ib)
 
@@ -440,17 +436,21 @@ def color_rgbw_to_rgb(r: int, g: int, b: int, w: int) -> tuple[int, int, int]:
 
 
 def color_rgb_to_rgbww(
-    r: int, g: int, b: int, min_mireds: int, max_mireds: int
+    r: int, g: int, b: int, min_kelvin: int, max_kelvin: int
 ) -> tuple[int, int, int, int, int]:
     """Convert an rgb color to an rgbww representation."""
     # Find the color temperature when both white channels have equal brightness
+    max_mireds = color_temperature_kelvin_to_mired(min_kelvin)
+    min_mireds = color_temperature_kelvin_to_mired(max_kelvin)
     mired_range = max_mireds - min_mireds
     mired_midpoint = min_mireds + mired_range / 2
     color_temp_kelvin = color_temperature_mired_to_kelvin(mired_midpoint)
     w_r, w_g, w_b = color_temperature_to_rgb(color_temp_kelvin)
 
     # Find the ratio of the midpoint white in the input rgb channels
-    white_level = min(r / w_r, g / w_g, b / w_b if w_b else 0)
+    white_level = min(
+        r / w_r if w_r else 0, g / w_g if w_g else 0, b / w_b if w_b else 0
+    )
 
     # Subtract the white portion from the rgb channels.
     rgb = (r - w_r * white_level, g - w_g * white_level, b - w_b * white_level)
@@ -462,17 +462,22 @@ def color_rgb_to_rgbww(
 
 
 def color_rgbww_to_rgb(
-    r: int, g: int, b: int, cw: int, ww: int, min_mireds: int, max_mireds: int
+    r: int, g: int, b: int, cw: int, ww: int, min_kelvin: int, max_kelvin: int
 ) -> tuple[int, int, int]:
     """Convert an rgbww color to an rgb representation."""
     # Calculate color temperature of the white channels
+    max_mireds = color_temperature_kelvin_to_mired(min_kelvin)
+    min_mireds = color_temperature_kelvin_to_mired(max_kelvin)
     mired_range = max_mireds - min_mireds
     try:
         ct_ratio = ww / (cw + ww)
     except ZeroDivisionError:
         ct_ratio = 0.5
     color_temp_mired = min_mireds + ct_ratio * mired_range
-    color_temp_kelvin = color_temperature_mired_to_kelvin(color_temp_mired)
+    if color_temp_mired:
+        color_temp_kelvin = color_temperature_mired_to_kelvin(color_temp_mired)
+    else:
+        color_temp_kelvin = 0
     w_r, w_g, w_b = color_temperature_to_rgb(color_temp_kelvin)
     white_level = max(cw, ww) / 255
 
@@ -528,6 +533,53 @@ def color_temperature_to_rgb(
     return red, green, blue
 
 
+def color_temperature_to_rgbww(
+    temperature: int, brightness: int, min_kelvin: int, max_kelvin: int
+) -> tuple[int, int, int, int, int]:
+    """Convert color temperature in kelvin to rgbcw.
+
+    Returns a (r, g, b, cw, ww) tuple.
+    """
+    max_mireds = color_temperature_kelvin_to_mired(min_kelvin)
+    min_mireds = color_temperature_kelvin_to_mired(max_kelvin)
+    temperature = color_temperature_kelvin_to_mired(temperature)
+    mired_range = max_mireds - min_mireds
+    cold = ((max_mireds - temperature) / mired_range) * brightness
+    warm = brightness - cold
+    return (0, 0, 0, round(cold), round(warm))
+
+
+def rgbww_to_color_temperature(
+    rgbww: tuple[int, int, int, int, int], min_kelvin: int, max_kelvin: int
+) -> tuple[int, int]:
+    """Convert rgbcw to color temperature in kelvin.
+
+    Returns a tuple (color_temperature, brightness).
+    """
+    _, _, _, cold, warm = rgbww
+    return _white_levels_to_color_temperature(cold, warm, min_kelvin, max_kelvin)
+
+
+def _white_levels_to_color_temperature(
+    cold: int, warm: int, min_kelvin: int, max_kelvin: int
+) -> tuple[int, int]:
+    """Convert whites to color temperature in kelvin.
+
+    Returns a tuple (color_temperature, brightness).
+    """
+    max_mireds = color_temperature_kelvin_to_mired(min_kelvin)
+    min_mireds = color_temperature_kelvin_to_mired(max_kelvin)
+    brightness = warm / 255 + cold / 255
+    if brightness == 0:
+        # Return the warmest color if brightness is 0
+        return (min_kelvin, 0)
+    return round(
+        color_temperature_mired_to_kelvin(
+            ((cold / 255 / brightness) * (min_mireds - max_mireds)) + max_mireds
+        )
+    ), min(255, round(brightness * 255))
+
+
 def _clamp(color_component: float, minimum: float = 0, maximum: float = 255) -> float:
     """
     Clamp the given color component value between the given min and max values.
